#include "loader.h"

#define __LOADER_C__
#include "loader_private.c"


/** Charge le code source en assembleur dans la machine
*   virtuel  partir d'un tableau de fichiers sources.
*   @param vm est la machine virtuelle utilise
*   @param files est un tableau de pointeurs sur fichiers
*   @param nopponents est le nombre de scripts  charger, ie la taille
*          du tableau files.
*/
OPPONENT_SIZE loadOpponentsFromFiles(VIRTUAL_MACHINE *vm, FILE **files, OPPONENT_SIZE nopponents)
{
    /* bloc_code est un tableau de nopponents chaines de caractres alloues dynamiquements */
    char **bloc_code = (char**)malloc(nopponents * sizeof(char*));

    /* code_final est un tableau de nopponents tableaux de MEMORY_ITEM allous dynamiquements */
    MEMORY_ITEM **code_final = (MEMORY_ITEM**)malloc(nopponents * sizeof(MEMORY_ITEM*));

    /* taille_bloc_code et taille_code_final sont respectivement les tailles de chacun des tableaux qui composent bloc_code et code_final  */
    CODE_SIZE *taille_bloc_code = (CODE_SIZE*)malloc(nopponents * sizeof(CODE_SIZE));
    CODE_SIZE *taille_code_final = (CODE_SIZE*)malloc(nopponents * sizeof(CODE_SIZE));

    unsigned short i=0;
    boolean estOK=TRUE;

    /* On boucle pour chaque fichier  charger */
    for(i=0;i<nopponents;i++)
    {
        /* On initialise tous les tableau  NULL */
        bloc_code[i]=NULL;
        code_final[i]=NULL;

        /* On charge dans bloc_code[i] la totalit du fichier source i */
        taille_bloc_code[i] = charger(&(bloc_code[i]), files[i]);
        /* Si une erreur se produit ou si le fichier est vide, alors on libre la mmoire, et on retourne 0 fichiers chargs */
        if(taille_bloc_code[i] == 0)
        {
            {
                unsigned short k=0;
                for(k=0;k<=i;k++)
                {
                    free(bloc_code[k]);
                    free(code_final[k]);
                }
                free(bloc_code);
                free(code_final);
                free(taille_bloc_code);
                free(taille_code_final);
            }
            return 0;
        }

        /* Si tout s'est bien droul, on formate le bloc prcdement charg afin de : supprimer les sauts de lignes, sparer les triplets par des underscores, rajouter le 0 dans le cas d'un doublet */
        taille_bloc_code[i] = formater(&(bloc_code[i]), taille_bloc_code[i]);
        /* Si une erreur se produit ou si le fichier est vide, alors on libre la mmoire, et on retourne 0 fichiers chargs */
        if(taille_bloc_code[i] == 0)
        {
            {
                unsigned short k=0;
                for(k=0;k<=i;k++)
                {
                    free(bloc_code[k]);
                    free(code_final[k]);
                }
                free(bloc_code);
                free(code_final);
                free(taille_bloc_code);
                free(taille_code_final);
            }
            return 0;
        }

        /* Si tout s'est bien droul, on analyse la chaine format pour la transformer en un tableau de MEMORY_ITEM */
        taille_code_final[i] = transcrire(bloc_code[i], taille_bloc_code[i], &(code_final[i]));
        /* Si une erreur se produit ou si le fichier est vide, alors on libre la mmoire, et on retourne 0 fichiers chargs */
        if(taille_code_final[i] == 0)
        {
            {
                unsigned short k=0;
                for(k=0;k<=i;k++)
                {
                    free(bloc_code[k]);
                    free(code_final[k]);
                }
                free(bloc_code);
                free(code_final);
                free(taille_bloc_code);
                free(taille_code_final);
            }
            return 0;
        }

    }

    /* On affiche les codes de chaque joueur */
    ASM_displayCodes(code_final,taille_code_final,nopponents);

    /* Si tout s'est bien droul, on charge les tableaux de MEMORY_ITEM et leurs tailles dans la machine virtuel puis on libre la mmoire */
    estOK = VM_secureOpponentLoading(vm, nopponents, code_final, taille_code_final);
    {
        unsigned short k=0;
        for(k=0;k<nopponents;k++)
        {
            free(bloc_code[k]);
            free(code_final[k]);
        }
        free(bloc_code);
        free(code_final);
        free(taille_bloc_code);
        free(taille_code_final);
    }

    /* Si le chargement s'est droul sans problme nopponents fichiers ont t chargs */
    return (estOK?nopponents:0);
}


