#ifndef __LOADER__
#define __LOADER__

#include <stdio.h>
#include <stdlib.h>
#include "types.h"
#include "vm.h"
#include "string.h"
#include "boolean.h"
#include "interpreter.h"

/** Charge le code source en assembleur dans la machine
*   virtuel  partir d'un tableau de fichiers sources.
*   @param vm est la machine virtuelle utilise
*   @param files est un tableau de pointeurs sur fichiers
*   @param nopponents est le nombre de scripts  charger, ie la taille
*          du tableau files.
*/
OPPONENT_SIZE loadOpponentsFromFiles(VIRTUAL_MACHINE *vm, FILE **files, OPPONENT_SIZE nopponents);

/** Charge la totalit d'un fichier dans une chaine de caractre.
*   La chaine bloc_code est allou dans la fonction charger. Renvois 0 si une erreur
*   se produit.
*   @param bloc_code est l'adresse d'une chaine de caratres.
*   @param fichier le fichier d'o que l'on stock dans bloc_code.
*/
CODE_SIZE charger(char **bloc_code, FILE *fichier);

/** Formate la chaine bloc_code de la faon suivante : un espace entre les paramtres,
*   un '_' pour sparer les triplets (instruction param1 param2). La fonction place un 0
*   si un paramtre venait  ne pas tre explicitement crit. La fonction redimensionne
*   la chaine, elle renvoie sa nouvelle taille. Renvois 0 si une erreur
*   se produit.
*   @param bloc_code est l'adresse de la chaine de caratres  formater.
*   @param taille est la taille de cette chaine
*/
CODE_SIZE formater(char **bloc_code, CODE_SIZE taille);

/** Analyse la chaine bloc_code et remplit le tableau code_final. La fonction transforme
*   chaque instruction en un MEMORY_ITEM suivi de 2 paramtres. On utilise 0 si la fonction
*   a besoin de moins de 2 paramtres. On alloue code_final dans la fonction. Renvois 0 si une erreur
*   se produit.
*   @param bloc_code la chaine source
*   @param taille la taille de la chaine source
*   @param code final l'adresse d'un tableau de MEMORY_ITEM
*/
CODE_SIZE transcrire(char *bloc_code, CODE_SIZE taille, MEMORY_ITEM **code_final);

/** Calcule le nombre de triplets (instruction param1 param2) qui composent une
*   chaine pralablement formate.
*   @param code est une chaine de caratres.
*/
CODE_SIZE nbinst(char *code);

/** Calcule a puissance b
*   @param a
*   @param b
*/
MEMORY_ITEM power(long a, short b);

/** Renvois le nombre correspondant  une chaine de caractre de type nombre.
*   Si la fonction rencontre une lettre elle renvois 0.
*   @param nombre une chaine de caractre.
*/
MEMORY_ITEM valeur(char *nombre);

/** Renvois TRUE si le caractere est un chiffre, FALSE sinon.
*   @param caractere  tester
*/
boolean chiffre(char caractere);

/** Renvois TRUE si le caractere est une lettre, FALSE sinon.
*   @param caractere  tester
*/
boolean lettre(char caractere);

#endif

