#ifndef __LOADER_C__
#error Cant compile loader_private.c outside loader.c
#endif

#define NB_INST_CONNUES 28

/** Calcule le nombre de triplets (instruction param1 param2) qui composent une
*   chaine pralablement formate.
*   @param code est une chaine de caratres.
*/
CODE_SIZE nbinst(char *code)
{
    CODE_SIZE nb = 0;
    unsigned short i=0;

    while(code[i] != '\0')
    {
        /* ' _ ' est le sparateur des triplets */
        if(code[i]=='_')
        {
            nb++;
        }
        i++;
    }

    return nb+1;
}

/** Calcule a puissance b
*   @param a
*   @param b
*/
MEMORY_ITEM power(long a, short b)
{

    MEMORY_ITEM result=1;
    unsigned short i;
    if(b==0)
    {
        return 1;
    }
    else
    {
        for(i=0;i<b;i++)
        {
            result = result * a;
        }
    }
    return result;
}

/** Renvois le nombre correspondant  une chaine de caractre de type nombre.
*   Si la fonction rencontre une lettre elle renvois 0.
*   @param nombre une chaine de caractre.
*/
MEMORY_ITEM valeur(char *nombre)
{
    unsigned short taille = strlen(nombre);
    MEMORY_ITEM valeur = 0;
    int i=0;
    while(nombre != NULL && taille>0)
    {
        switch(nombre[i])
        {
            case '0' :
            break;
            case '1' :
            valeur=valeur+(power(10,(taille-1)));
            break;
            case '2' :
            valeur=valeur+(2*power(10,(taille-1)));
            break;
            case '3' :
            valeur=valeur+(3*power(10,(taille-1)));
            break;
            case '4' :
            valeur=valeur+(4*power(10,(taille-1)));
            break;
            case '5' :
            valeur=valeur+(5*power(10,(taille-1)));
            break;
            case '6' :
            valeur=valeur+(6*power(10,(taille-1)));
            break;
            case '7' :
            valeur=valeur+(7*power(10,(taille-1)));
            break;
            case '8' :
            valeur=valeur+(8*power(10,(taille-1)));
            break;
            case '9' :
            valeur=valeur+(9*power(10,(taille-1)));
            break;
            default  :
            return 0;
            break;
        }
        taille--;
        i++;
    }

    return valeur;
}

/** Renvois TRUE si le caractere est un chiffre, FALSE sinon.
*   @param caractere  tester
*/
boolean chiffre(char caractere)
{
    if(caractere<'0' || caractere>'9')
    {
        return FALSE;
    }
    else
    {
        return TRUE;
    }
}

/** Renvois TRUE si le caractere est une lettre, FALSE sinon.
*   @param caractere  tester
*/
boolean lettre(char caractere)
{
    if(caractere<'A' || (caractere>'Z' && caractere<'a') || caractere>'z')
    {
        return FALSE;
    }
    else
    {
        return TRUE;
    }
}

/** Charge la totalit d'un fichier dans une chaine de caractre.
*   La chaine bloc_code est allou dans la fonction charger. Renvois 0 si une erreur
*   se produit.
*   @param bloc_code est l'adresse d'une chaine de caratres.
*   @param fichier le fichier que l'on stock dans bloc_code.
*/
CODE_SIZE charger(char **bloc_code, FILE *fichier)
{
    char buffer[1024];
    CODE_SIZE nblu=0, i=0, j=0;

    /* On vrifie que le fichier est ouvert correctement */
    if(fichier==NULL)
    {
        fprintf(stderr,"fonction charger : ouverture de fichier impossible");
        return 0;
    }

    /* On s'assure que le curseur est au debut du fichier */
    rewind(fichier);

    /* On charge une premiere fois le buffer */
    nblu = fread(buffer, sizeof(char), 1024, fichier);

    /* Tant que le buffer est completement charg, on augmente la taille de *bloc_code et on copie le buffer dedans */
    while(nblu == 1024)
    {
        *bloc_code = (char*)realloc(*bloc_code,((i+1024)*sizeof(char)));
        for(j=0; j<1024; j++)
        {
            (*bloc_code)[i+j] = buffer[j];
        }
        i = i+1024;
        nblu = fread(buffer, sizeof(char), 1024, fichier);
    }
    /* Une fois la fin du dernier fichier atteint, on augmente la taille de *bloc_code et on vide ce qu'il reste dans le buffer */
    *bloc_code = (char*)realloc(*bloc_code,((i+nblu+1)*sizeof(char)));
    for(j=0; j<nblu; j++)
    {
        (*bloc_code)[i+j] = buffer[j];
    }
    (*bloc_code)[i+j] = '\0';

    /* On retourne la taille de la chaine final sans compter le \0 */
    return (i+j);
}

/** Formate la chaine bloc_code de la faon suivante : un espace entre les paramtres,
*   un '_' pour sparer les triplets (instruction param1 param2). La fonction place un 0
*   si un paramtre venait  ne pas tre explicitement crit. La fonction redimensionne
*   la chaine, elle renvoie sa nouvelle taille. Renvois 0 si une erreur
*   se produit.
*   @param bloc_code est l'adresse de la chaine de caratres  formater.
*   @param taille est la taille de cette chaine
*/
CODE_SIZE formater(char **bloc_code, CODE_SIZE taille)
{
    char *bloc_nouveau=NULL;
    CODE_SIZE i=0;
    CODE_SIZE j=0;
    boolean Comm=FALSE, Adr1=FALSE, Adr2=FALSE;

    /*On avance dans la chaine tant qu'on n'a pas une lettre*/
    while(i<taille && !lettre((*bloc_code)[i]))
    {
        i++;
    }

    /*On alloue une place en memoire pour bloc_nouveau et on copie le premier caractere lettre de bloc_code*/
    if(i<taille)
    {
        bloc_nouveau=(char*)malloc(sizeof(char));
        if(bloc_nouveau==NULL)
        {
            fprintf(stderr,"fonction formater : allocation impossible");
            return 0;
        }
        bloc_nouveau[j]=(*bloc_code)[i];
        i++;
        j++;
        Comm=TRUE;
    }

    while(i<taille)
    {

        /*On avance tant qu'on n'a pas une lettre*/
        while(i<taille && !lettre((*bloc_code)[i]))
        {
            i++;
        }

        /*Tant qu'on a des lettres on copie bloc_code dans bloc_nouveau en agrandissant bloc_nouveau*/
        while(i<taille && lettre((*bloc_code)[i]))
        {
            bloc_nouveau=(char*)realloc(bloc_nouveau, (j+1)*sizeof(char));
            if(bloc_nouveau==NULL)
            {
                fprintf(stderr,"fonction charger : reallocation impossible");
                return 0;
            }
            bloc_nouveau[j]=(*bloc_code)[i];
            i++;
            j++;
            Comm=TRUE;
        }

        /*Si on a une commande on ajoute a bloc_nouveau un espace*/
        if(Comm==TRUE)
        {
            bloc_nouveau=(char*)realloc(bloc_nouveau, (j+1)*sizeof(char));
            if(bloc_nouveau==NULL)
            {
                fprintf(stderr,"fonction charger : reallocation impossible");
                return 0;
            }
            bloc_nouveau[j]=' ';
            j++;
        }

        /*On avance tant qu'on n'a pas une lettre ou un chiffre*/
        while(i<taille && !(lettre((*bloc_code)[i]) || chiffre((*bloc_code)[i])))
        {
            i++;
        }

        /*Tant qu'on a un chiffre on copie bloc_code dans bloc_nouveau*/
        while(i<taille && chiffre((*bloc_code)[i]))
        {
            bloc_nouveau=(char*)realloc(bloc_nouveau, (j+1)*sizeof(char));
            if(bloc_nouveau==NULL)
            {
                fprintf(stderr,"fonction charger : reallocation impossible");
                return 0;
            }
            bloc_nouveau[j]=(*bloc_code)[i];
            i++;
            j++;
            Adr1=TRUE;
        }

        /*Si bloc_code comportait au moins un chiffre on ajoute un espace a bloc_nouveau*/
        if(Comm==TRUE && Adr1==TRUE)
        {
            bloc_nouveau=(char*)realloc(bloc_nouveau, (j+1)*sizeof(char));
            if(bloc_nouveau==NULL)
            {
                fprintf(stderr,"fonction charger : reallocation impossible");
                return 0;
            }
            bloc_nouveau[j]=' ';
            j++;
            Adr1=FALSE;
        }
        /*Sinon on ajoute 0 et un espace a bloc_nouveau*/
        else if(Comm==TRUE)
        {
            bloc_nouveau=(char*)realloc(bloc_nouveau, (j+2)*sizeof(char));
            if(bloc_nouveau==NULL)
            {
                fprintf(stderr,"fonction charger : reallocation impossible");
                return 0;
            }
            bloc_nouveau[j]='0';
            j++;
            bloc_nouveau[j]=' ';
            j++;
        }

        /*On avance tant qu'on n'a pas une lettre ou un chiffre*/
        while(i<taille && !(lettre((*bloc_code)[i]) || chiffre((*bloc_code)[i])))
        {
            i++;
        }

        /*Tant qu'on a un chiffre on copie bloc_code dans bloc_nouveau*/
        while(i<taille && chiffre((*bloc_code)[i]))
        {
            bloc_nouveau=(char*)realloc(bloc_nouveau, (j+1)*sizeof(char));
            if(bloc_nouveau==NULL)
            {
                fprintf(stderr,"fonction charger : reallocation impossible");
                return 0;
            }
            bloc_nouveau[j]=(*bloc_code)[i];
            i++;
            j++;
            Adr2=TRUE;
        }

        /*Si bloc_code comportait au moins un chiffre on ajoute un espace a bloc_nouveau*/
        if(Comm==TRUE && Adr2==TRUE)
        {
            bloc_nouveau=(char*)realloc(bloc_nouveau, (j+1)*sizeof(char));
            if(bloc_nouveau==NULL)
            {
                fprintf(stderr,"fonction charger : reallocation impossible");
                return 0;
            }
            bloc_nouveau[j]='_';
            j++;
            Adr2=FALSE;
        }
        /*Sinon on ajoute 0 et un underscore a bloc_nouveau*/
        else if(Comm==TRUE)
        {
            bloc_nouveau=(char*)realloc(bloc_nouveau, (j+2)*sizeof(char));
            if(bloc_nouveau==NULL)
            {
                fprintf(stderr,"fonction charger : reallocation impossible");
                return 0;
            }
            bloc_nouveau[j]='0';
            j++;
            bloc_nouveau[j]='_';
            j++;
        }
        Comm=FALSE;
        /*On a une instructin complete avec 2 parametre, on boucle jusqu'a ce qu'on soit a la fin de bloc_code*/
    }

    bloc_nouveau[j-1]='\0';
    /*On copie bloc_nouveau dans bloc_code et on libere bloc_nouveau*/
    free(*bloc_code);
    *bloc_code=(char*)malloc(j*sizeof(char));

    for(i=0; i<j; i++)
    {
        (*bloc_code)[i]=bloc_nouveau[i];
    }

    free(bloc_nouveau);

    /*On retourne le nombre de caracteres de la chaine bloc_code sans compter le \0*/
    return (j-1);
}

/** Analyse la chaine bloc_code et remplit le tableau code_final. La fonction transforme
*   chaque instruction en un MEMORY_ITEM suivi de 2 paramtres. On utilise 0 si la fonction
*   a besoin de moins de 2 paramtres. On alloue code_final dans la fonction. Renvois 0 si une erreur
*   se produit.
*   @param bloc_code la chaine source
*   @param taille la taille de la chaine source
*   @param code final l'adresse d'un tableau de MEMORY_ITEM
*/
CODE_SIZE transcrire(char *bloc_code, CODE_SIZE taille, MEMORY_ITEM **code_final)
{
    char *ptr_code = NULL;
    boolean found = FALSE;
    unsigned short i=0,j=0,k=0,l=0;

    if(bloc_code == NULL)
    {
        return 0;
    }

    /* La liste des instructions connues : NB_INST_CONNUES est dfinie plus haut */
    char instruction[][15]={
                               "NOP",
                               "NOOP",
                               "DECLARE",
                               "MOVE",
                               "MOVA",
                               "LESSER",
                               "GREATER",
                               "EQUAL",
                               "ZERO",
                               "JUMP",
                               "JUMPIF",
                               "JUMPIFNOT",
                               "INC",
                               "DEC",
                               "ADD",
                               "SUB",
                               "MUL",
                               "DIV",
                               "MOD",
                               "RAND",
                               "MEMSIZE",
                               "CODESEG",
                               "REGSEG",
                               "CO",
                               "DISPVAL",
                               "DISP",
                               "DISPSTR",
                               "END"
                           };

    /* Respectivement le nombre de paramtre que prend chacune des instructions connues */
    unsigned short nbParam[]=
        {
            0,0,1,2,2,2,2,2,1,1,1,1,1,1,2,2,2,2,2,1,1,2,2,1,1,1,2,0
        };

    /* La taille du code final sera gale  3 fois le nombre de triplet (inst param1 param2) */
    CODE_SIZE taille_code_final = 3 * nbinst(bloc_code);

    /* On alloue le tableau de MEMORY_ITEM pass en paramtre qui contiendra le code final */
    *code_final = (MEMORY_ITEM*)malloc(taille_code_final*sizeof(MEMORY_ITEM));
    if(*code_final == NULL)
    {
        fprintf(stderr,"fonction transcrire : reallocation impossible");
        return 0;
    }

    /* On initialise ptr_code sur la premiere chaine allant jusqu'au premier separateur espace ou underscore */
    ptr_code = strtok(bloc_code, " _");

    while(ptr_code != NULL)
    {
        for(i=0; !found && i < NB_INST_CONNUES; i++)
        {
            /* On test pour chaque instruction connue si on peu trouver une correspondance au code */
            if(compareStrings(ptr_code, instruction[i],15)==0)
            {
                found = TRUE;
                (*code_final)[k] = i;
                k++;
            }
        }

        if(found == FALSE)
        {
            /* Si on a rien pu trouver il y a erreur de syntaxe dans le fichier asm, on retourne une erreur */
            printf("Instruction inconnue");
            return 0;
        }

        /* On recherche les paramtres de l'instruction trouv */
        for(j=1; j <= nbParam[(*code_final)[k-j]] ; j++)
        {
            ptr_code = strtok(NULL, " _");
            (*code_final)[k] = valeur(ptr_code);
            k++;
        }

        /* On complte par des 0 s'il y a moins de 2 paramtres ncssaires */
        for(l=j; l <= 2; l++)
        {
            ptr_code = strtok(NULL, " _");
            (*code_final)[k] = 0;
            k++;
        }

        /* On passe a l'instruction suivante */
        ptr_code = strtok(NULL, " _");
        found = FALSE;
    }

    return taille_code_final;
}

